@@ -56,13 +56,24 @@ for dir in Custommarshalers \
 		  System.Data \
 		  Mono.Data.PostgreSqlClient \
 		  Mono.Data.MySql \
+		  Mono.Data.SqliteClient \
+		  Mono.Data.SybaseClient \
+		  Mono.Data.TdsClient \
 		  ByteFX.Data \
 		  Npgsql \
 		  Mono.Data.DB2Client \
 		  System.Data.OracleClient \
+		  System.Data.SqlXml \
+		  System.Data.ObjectSpaces \
 		  Mono.Directory.LDAP \
 		  Mono.GetOptions \
-		  Mono.Posix ; do
+		  Mono.Posix \
+		  Mono.Cairo \
+		  Novell.Directory.Ldap \
+		  System.DirectoryServices \
+		  Mono.Security \
+		  Mono.Security.Win32 \
+		  System.Security ; do
 
 	cp -pr "$MCS_SOURCES/class/$dir" "mcs-sources/class/$dir"
 
