@@ -36,7 +36,11 @@ namespace System.Data {
 		private string rowError;
 		private DataRowState rowState;
 		internal int xmlRowID = 0;
+		internal bool _nullConstraintViolation;
+		private string _nullConstraintMessage;
 		private bool editing = false;
+		private bool _hasParentCollection;
+		private bool _inChangingEvent;
 
 		#endregion
 
@@ -51,27 +55,25 @@ namespace System.Data {
 			_table = builder.Table;
 
 			original = null; 
-			current = new object[_table.Columns.Count];
-			// initialize to DBNull.Value
-			for (int c = 0; c < _table.Columns.Count; c++) {
-				current[c] = DBNull.Value;
-			}
+			
 			proposed = new object[_table.Columns.Count];
-			Array.Copy (current, proposed, _table.Columns.Count);
-
+			for (int c = 0; c < _table.Columns.Count; c++) 
+			{
+				proposed[c] = DBNull.Value;
+			}
+			
 			columnErrors = new string[_table.Columns.Count];
 			rowError = String.Empty;
 
 			//on first creating a DataRow it is always detached.
 			rowState = DataRowState.Detached;
-
+			
 			foreach (DataColumn Col in _table.Columns) {
 				
 				if (Col.AutoIncrement) {
 					this [Col] = Col.AutoIncrementValue();
 				}
 			}
-
 			_table.Columns.CollectionChanged += new System.ComponentModel.CollectionChangeEventHandler(CollectionChanged);
 		}
 
@@ -86,7 +88,15 @@ namespace System.Data {
 		public bool HasErrors {
 			[MonoTODO]
 			get {
-				throw new NotImplementedException ();
+				if (RowError != string.Empty)
+					return true;
+
+				for (int i= 0; i < columnErrors.Length; i++){
+					if (columnErrors[i] != null && columnErrors[i] != string.Empty)
+						return true;
+				}
+
+				return false;
 			}
 		}
 
@@ -107,7 +117,9 @@ namespace System.Data {
 		/// Gets or sets the data stored in specified DataColumn
 		/// </summary>
 		public object this[DataColumn column] {
-			get { return this[column, DataRowVersion.Default]; } 
+
+			get {
+				return this[column, DataRowVersion.Default];} 
 			set {
 				int columnIndex = _table.Columns.IndexOf (column);
 				if (columnIndex == -1)
@@ -129,6 +141,7 @@ namespace System.Data {
 				DataColumn column = _table.Columns[columnIndex];
 				_table.ChangingDataColumn (this, column, value);
 				
+				
 				bool orginalEditing = editing;
 				if (!orginalEditing) BeginEdit ();
 				object v = SetColumnValue (value, columnIndex);
@@ -170,26 +183,37 @@ namespace System.Data {
 			get {
 				if (columnIndex < 0 || columnIndex > _table.Columns.Count)
 					throw new IndexOutOfRangeException ();
-				// Non-existent version
-				if (rowState == DataRowState.Detached && version == DataRowVersion.Current || !HasVersion (version))
-					throw new VersionNotFoundException (Locale.GetText ("There is no " + version.ToString () + " data to access."));
 				// Accessing deleted rows
 				if (rowState == DataRowState.Deleted && version != DataRowVersion.Original)
 					throw new DeletedRowInaccessibleException ("Deleted row information cannot be accessed through the row.");
-				switch (version) {
-				case DataRowVersion.Default:
-					if (editing || rowState == DataRowState.Detached)
-						return proposed[columnIndex];
-					return current[columnIndex];
-				case DataRowVersion.Proposed:
-					return proposed[columnIndex];
-				case DataRowVersion.Current:
-					return current[columnIndex];
-				case DataRowVersion.Original:
-					return original[columnIndex];
-				default:
-					throw new ArgumentException ();
+				// Row not in table
+				//if (rowState == DataRowState.Detached && version == DataRowVersion.Default)
+				//	throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
+				// Non-existent version
+				//if (!HasVersion (version))
+				//	throw new VersionNotFoundException (Locale.GetText ("There is no " + version.ToString () + " data to access."));
+				if (HasVersion(version))
+				{
+					switch (version) 
+					{
+						case DataRowVersion.Default:
+							if (editing || rowState == DataRowState.Detached)
+								return proposed[columnIndex];
+							return current[columnIndex];
+						case DataRowVersion.Proposed:
+							return proposed[columnIndex];
+						case DataRowVersion.Current:
+							return current[columnIndex];
+						case DataRowVersion.Original:
+							return original[columnIndex];
+						default:
+							throw new ArgumentException ();
+					}
 				}
+				if (rowState == DataRowState.Detached && proposed == null)
+					throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
+				
+				throw new VersionNotFoundException (Locale.GetText ("There is no " + version.ToString () + " data to access."));
 			}
 		}
 
@@ -199,6 +223,13 @@ namespace System.Data {
 		[MonoTODO]
 		public object[] ItemArray {
 			get { 
+				// row not in table
+				if (rowState == DataRowState.Detached)
+					throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
+				// Accessing deleted rows
+				if (rowState == DataRowState.Deleted)
+					throw new DeletedRowInaccessibleException ("Deleted row information cannot be accessed through the row.");
+				
 				return current; 
 			}
 			set {
@@ -232,96 +263,114 @@ namespace System.Data {
 			object newval = null;
 			DataColumn col = _table.Columns[index];
 			
-			if (col.ReadOnly && v != this[index])
+			if (_hasParentCollection && col.ReadOnly && v != this[index])
 				throw new ReadOnlyException ();
 
-			if (v == null) {
-				if(col.DefaultValue != DBNull.Value) {
+			if (v == null)
+			{
+				if(col.DefaultValue != DBNull.Value) 
+				{
 					newval = col.DefaultValue;
 				}
- 				else if(col.AutoIncrement == true) {
+				else if(col.AutoIncrement == true) 
+				{
 					newval = this [index];
- 				}
-				else {
+				}
+				else 
+				{
 					if (!col.AllowDBNull)
-						throw new NoNullAllowedException ();
+					{
+						//Constraint violations during data load is raise in DataTable EndLoad
+						this._nullConstraintViolation = true;
+						_nullConstraintMessage = "Column '" + col.ColumnName + "' does not allow nulls.";
+					}
+
 					newval = DBNull.Value;
 				}
-			}
-			else if (v == DBNull.Value) {
+			}		
+			else if (v == DBNull.Value) 
+			{
+				
 				if (!col.AllowDBNull)
-					throw new NoNullAllowedException ();
-				else {
-					newval = DBNull.Value;
+				{
+					//Constraint violations during data load is raise in DataTable EndLoad
+					this._nullConstraintViolation = true;
+					_nullConstraintMessage = "Column '" + col.ColumnName + "' does not allow nulls.";
 				}
+				
+				newval = DBNull.Value;
 			}
-			else {	
+			else 
+			{	
 				Type vType = v.GetType(); // data type of value
 				Type cType = col.DataType; // column data type
-				if (cType != vType) {
+				if (cType != vType) 
+				{
 					TypeCode typeCode = Type.GetTypeCode(cType);
 					switch(typeCode) {
-					case TypeCode.Boolean :
-						v = Convert.ToBoolean (v);
-						break;
-					case TypeCode.Byte  :
-						v = Convert.ToByte (v);
-						break;
-					case TypeCode.Char  :
-						v = Convert.ToChar (v);
-						break;
-					case TypeCode.DateTime  :
-						v = Convert.ToDateTime (v);
-						break;
-					case TypeCode.Decimal  :
-						v = Convert.ToDecimal (v);
-						break;
-					case TypeCode.Double  :
-						v = Convert.ToDouble (v);
-						break;
-					case TypeCode.Int16  :
-						v = Convert.ToInt16 (v);
-						break;
-					case TypeCode.Int32  :
-						v = Convert.ToInt32 (v);
-						break;
-					case TypeCode.Int64  :
-						v = Convert.ToInt64 (v);
-						break;
-					case TypeCode.SByte  :
-						v = Convert.ToSByte (v);
-						break;
-					case TypeCode.Single  :
-						v = Convert.ToSingle (v);
-						break;
-					case TypeCode.String  :
-						v = Convert.ToString (v);
-						break;
-					case TypeCode.UInt16  :
-						v = Convert.ToUInt16 (v);
-						break;
-					case TypeCode.UInt32  :
-						v = Convert.ToUInt32 (v);
-						break;
-					case TypeCode.UInt64  :
-						v = Convert.ToUInt64 (v);
-						break;
-					default :
+						case TypeCode.Boolean :
+							v = Convert.ToBoolean (v);
+							break;
+						case TypeCode.Byte  :
+							v = Convert.ToByte (v);
+							break;
+						case TypeCode.Char  :
+							v = Convert.ToChar (v);
+							break;
+						case TypeCode.DateTime  :
+							v = Convert.ToDateTime (v);
+							break;
+						case TypeCode.Decimal  :
+							v = Convert.ToDecimal (v);
+							break;
+						case TypeCode.Double  :
+							v = Convert.ToDouble (v);
+							break;
+						case TypeCode.Int16  :
+							v = Convert.ToInt16 (v);
+							break;
+						case TypeCode.Int32  :
+							v = Convert.ToInt32 (v);
+							break;
+						case TypeCode.Int64  :
+							v = Convert.ToInt64 (v);
+							break;
+						case TypeCode.SByte  :
+							v = Convert.ToSByte (v);
+							break;
+						case TypeCode.Single  :
+							v = Convert.ToSingle (v);
+							break;
+						case TypeCode.String  :
+							v = Convert.ToString (v);
+							break;
+						case TypeCode.UInt16  :
+							v = Convert.ToUInt16 (v);
+							break;
+						case TypeCode.UInt32  :
+							v = Convert.ToUInt32 (v);
+							break;
+						case TypeCode.UInt64  :
+							v = Convert.ToUInt64 (v);
+							break;
+						default :
+								
 						switch(cType.ToString()) {
-						case "System.TimeSpan" :
-							v = (System.TimeSpan) v;
-							break;
-						case "System.Type" :
-							v = (System.Type) v;
-							break;
-						case "System.Object" :
-							//v = (System.Object) v;
-							break;
-						default:
-							// FIXME: is exception correct?
-							throw new InvalidCastException("Type not supported.");
+							case "System.TimeSpan" :
+								v = (System.TimeSpan) v;
+								break;
+							case "System.Type" :
+								v = (System.Type) v;
+								break;
+							case "System.Object" :
+								//v = (System.Object) v;
+								break;
+							default:
+								if (!cType.IsArray)
+									throw new InvalidCastException("Type not supported.");
+								break;
 						}
-						break;
+							break;
 					}
 					vType = v.GetType();
 				}
@@ -363,6 +412,7 @@ namespace System.Data {
 		//from a Datatable so I added this method. Delete if there is a better way.
 		internal void DetachRow() {
 			proposed = null;
+			_hasParentCollection = false;
 			rowState = DataRowState.Detached;
 		}
 
@@ -399,7 +449,8 @@ namespace System.Data {
 				rowState = DataRowState.Unchanged;
 				break;
 			case DataRowState.Deleted:
-				_table.Rows.Remove (this);
+				_table.Rows.RemoveInternal (this);
+				DetachRow();
 				break;
 			case DataRowState.Detached:
 				throw new RowNotInTableException("Cannot perform this operation on a row not in the table.");
@@ -457,17 +508,140 @@ namespace System.Data {
 		[MonoTODO]
 		public void Delete () 
 		{
+			_table.DeletingDataRow(this, DataRowAction.Delete);
 			switch (rowState) {
 			case DataRowState.Added:
-				Table.Rows.Remove (this);
+				Table.Rows.RemoveInternal (this);
+				// if row was in Added state we move it to Detached.
+				DetachRow();
 				break;
 			case DataRowState.Deleted:
-				throw new DeletedRowInaccessibleException ();
+				break;
 			default:
-				//TODO: Events, Constraints
+				// check what to do with child rows
+				CheckChildRows(DataRowAction.Delete);
 				rowState = DataRowState.Deleted;
 				break;
 			}
+				_table.DeletedDataRow(this, DataRowAction.Delete);
+		}
+
+		// check the child rows of this row before deleting the row.
+		private void CheckChildRows(DataRowAction action)
+		{
+			
+			// in this method we find the row that this row is in a reltion with them.
+			// in shortly we find all child rows of this row.
+			// then we function according to the DeleteRule of the foriegnkey.
+
+
+			// 1. find if this row is attached to dataset.
+			// 2. find if EnforceConstraints is true.
+			// 3. find if there are any constraint on the table that the row is in.
+			if (_table.DataSet != null && _table.DataSet.EnforceConstraints && _table.Constraints.Count > 0)
+			{
+				foreach (DataTable table in _table.DataSet.Tables)
+				{
+					// loop on all constraints of the table.
+					ConstraintCollection constraintsCollection = table.Constraints;
+					for (int i = 0; i < constraintsCollection.Count; i++)
+					{
+						ForeignKeyConstraint fk = null;
+						if (constraintsCollection[i] is ForeignKeyConstraint)
+						{
+							fk = (ForeignKeyConstraint)constraintsCollection[i];
+							if (fk.RelatedTable == _table)
+							{
+								//we create a dummy relation because we do not want to duplicate code of GetChild().
+								// we use the dummy relation to find child rows.
+								DataRelation rel = new DataRelation("dummy", fk.RelatedColumns, fk.Columns, false);
+								Rule rule;
+								if (action == DataRowAction.Delete)
+									rule = fk.DeleteRule;
+								else
+									rule = fk.UpdateRule;
+								CheckChildRows(rel, action, rule);
+							}
+						}			
+					}
+				}
+			}
+		}
+
+		private void CheckChildRows(DataRelation rel, DataRowAction action, Rule rule)
+		{				
+			DataRow[] childRows = GetChildRows(rel);
+			switch (rule)
+			{
+				case Rule.Cascade:  // delete or change all relted rows.
+					if (childRows != null)
+					{
+						for (int j = 0; j < childRows.Length; j++)
+						{
+							// if action is delete we delete all child rows
+							if (action == DataRowAction.Delete)
+							{
+								if (childRows[j].RowState != DataRowState.Deleted)
+									childRows[j].Delete();
+							}
+							// if action is change we change the values in the child row
+							else if (action == DataRowAction.Change)
+							{
+								// change only the values in the key columns
+								// set the childcolumn value to the new parent row value
+								for (int k = 0; k < rel.ChildColumns.Length; k++)
+									childRows[j][rel.ChildColumns[k]] = this[rel.ParentColumns[k], DataRowVersion.Proposed];
+							}
+						}
+					}
+					break;
+				case Rule.None: // throw an exception if there are any child rows.
+					if (childRows != null)
+					{
+						for (int j = 0; j < childRows.Length; j++)
+						{
+							if (childRows[j].RowState != DataRowState.Deleted)
+							{
+								string changeStr = "Cannot change this row because constraints are enforced on relation " + rel.RelationName +", and changing this row will strand child rows.";
+								string delStr = "Cannot delete this row because constraints are enforced on relation " + rel.RelationName +", and deleting this row will strand child rows.";
+								string message = action == DataRowAction.Delete ? delStr : changeStr;
+								throw new InvalidConstraintException(message);
+							}
+						}
+					}
+					break;
+				case Rule.SetDefault: // set the values in the child rows to the defult value of the columns.
+					if (childRows != null)
+					{
+						for (int j = 0; j < childRows.Length; j++)
+						{
+							DataRow child = childRows[j];
+							if (childRows[j].RowState != DataRowState.Deleted)
+							{
+								//set only the key columns to default
+								for (int k = 0; k < rel.ChildColumns.Length; k++)
+									child[rel.ChildColumns[k]] = rel.ChildColumns[k].DefaultValue;
+							}
+						}
+					}
+					break;
+				case Rule.SetNull: // set the values in the child row to null.
+					if (childRows != null)
+					{
+						for (int j = 0; j < childRows.Length; j++)
+						{
+							DataRow child = childRows[j];
+							if (childRows[j].RowState != DataRowState.Deleted)
+							{
+								// set only the key columns to DBNull
+								for (int k = 0; k < rel.ChildColumns.Length; k++)
+									child.SetNull(rel.ChildColumns[k]);
+							}
+						}
+					}
+					break;
+			}
+
 		}
 
 		/// <summary>
@@ -476,19 +650,47 @@ namespace System.Data {
 		[MonoTODO]
 		public void EndEdit () 
 		{
-			editing = false;
+			if (_inChangingEvent)
+				throw new InRowChangingEventException("Cannot call EndEdit inside an OnRowChanging event.");
 			if (rowState == DataRowState.Detached)
+			{
+				editing = false;
 				return;
+			}
+
 			if (HasVersion (DataRowVersion.Proposed))
 			{
+				_inChangingEvent = true;
+				try
+				{
+					_table.ChangingDataRow(this, DataRowAction.Change);
+				}
+				finally
+				{
+					_inChangingEvent = false;
+				}
 				if (rowState == DataRowState.Unchanged)
 					rowState = DataRowState.Modified;
 				
 				//Calling next method validates UniqueConstraints
 				//and ForeignKeys.
-				_table.Rows.ValidateDataRowInternal(this);
+				try
+				{
+					if ((_table.DataSet == null || _table.DataSet.EnforceConstraints) && !_table._duringDataLoad)
+						_table.Rows.ValidateDataRowInternal(this);
+				}
+				catch (Exception e)
+				{
+					editing = false;
+					proposed = null;
+					throw e;
+				}
+				// check all child rows.
+				CheckChildRows(DataRowAction.Change);
 				current = proposed;
 				proposed = null;
+				editing = false;
+				_table.ChangedDataRow(this, DataRowAction.Change);
 			}
 		}
 
@@ -515,23 +717,44 @@ namespace System.Data {
 		/// </summary>
 		public DataRow[] GetChildRows (DataRelation relation, DataRowVersion version) 
 		{
+			if (relation == null)
+				return new DataRow[0];
+
+			if (this.Table == null || RowState == DataRowState.Detached)
+				throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
+
+			if (relation.DataSet != this.Table.DataSet)
+				throw new ArgumentException();
+
 			// TODO: Caching for better preformance
 			ArrayList rows = new ArrayList();
 			DataColumn[] parentColumns = relation.ParentColumns;
 			DataColumn[] childColumns = relation.ChildColumns;
 			int numColumn = parentColumns.Length;
-			foreach (DataRow row in relation.ChildTable.Rows) {
-				bool allColumnsMatch = true;
-				for (int columnCnt = 0; columnCnt < numColumn; ++columnCnt) {
-					if (!this[parentColumns[columnCnt], version].Equals(
-					    row[childColumns[columnCnt], version])) {
-						allColumnsMatch = false;
-						break;
+			if (HasVersion(version)) 
+			{
+				foreach (DataRow row in relation.ChildTable.Rows) 
+				{
+					bool allColumnsMatch = false;
+					if (row.HasVersion(DataRowVersion.Default))
+					{
+						allColumnsMatch = true;
+						for (int columnCnt = 0; columnCnt < numColumn; ++columnCnt) 
+						{
+							if (!this[parentColumns[columnCnt], version].Equals(
+								row[childColumns[columnCnt], DataRowVersion.Default])) 
+							{
+								allColumnsMatch = false;
+								break;
+							}
+						}
 					}
+					if (allColumnsMatch) rows.Add(row);
 				}
-				if (allColumnsMatch) rows.Add(row);
 			}
-			return rows.ToArray(typeof(DataRow)) as DataRow[];
+			DataRow[] result = relation.ChildTable.NewRowArray(rows.Count);
+			rows.CopyTo(result, 0);
+			return result;
 		}
 
 		/// <summary>
@@ -559,7 +782,10 @@ namespace System.Data {
 			if (columnIndex < 0 || columnIndex >= columnErrors.Length)
 				throw new IndexOutOfRangeException ();
 
-			return columnErrors[columnIndex];
+			string retVal = columnErrors[columnIndex];
+			if (retVal == null)
+				retVal = string.Empty;
+			return retVal;
 		}
 
 		/// <summary>
@@ -579,11 +805,11 @@ namespace System.Data {
 
 			for (int i = 0; i < columnErrors.Length; i += 1)
 			{
-				if (columnErrors[i] != String.Empty)
+				if (columnErrors[i] != null && columnErrors[i] != String.Empty)
 					dataColumns.Add (_table.Columns[i]);
 			}
 
-			return (DataColumn[])(dataColumns.ToArray ());
+			return (DataColumn[])(dataColumns.ToArray (typeof(DataColumn)));
 		}
 
 		/// <summary>
@@ -647,22 +873,44 @@ namespace System.Data {
 		public DataRow[] GetParentRows (DataRelation relation, DataRowVersion version) 
 		{
 			// TODO: Caching for better preformance
+			if (relation == null)
+				return new DataRow[0];
+
+			if (this.Table == null || RowState == DataRowState.Detached)
+				throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
+
+			if (relation.DataSet != this.Table.DataSet)
+				throw new ArgumentException();
+
 			ArrayList rows = new ArrayList();
 			DataColumn[] parentColumns = relation.ParentColumns;
 			DataColumn[] childColumns = relation.ChildColumns;
 			int numColumn = parentColumns.Length;
-			foreach (DataRow row in relation.ParentTable.Rows) {
-				bool allColumnsMatch = true;
-				for (int columnCnt = 0; columnCnt < numColumn; ++columnCnt) {
-					if (!this[parentColumns[columnCnt], version].Equals(
-					    row[childColumns[columnCnt], version])) {
-						allColumnsMatch = false;
-						break;
+			if (HasVersion(version))
+			{
+				foreach (DataRow row in relation.ParentTable.Rows) 
+				{
+					bool allColumnsMatch = false;
+					if (row.HasVersion(DataRowVersion.Default))
+					{
+						allColumnsMatch = true;
+						for (int columnCnt = 0; columnCnt < numColumn; columnCnt++) 
+						{
+							if (!this[childColumns[columnCnt], version].Equals(
+								row[parentColumns[columnCnt], DataRowVersion.Default])) 
+							{
+								allColumnsMatch = false;
+								break;
+							}
+						}
 					}
+					if (allColumnsMatch) rows.Add(row);
 				}
-				if (allColumnsMatch) rows.Add(row);
 			}
-			return rows.ToArray(typeof(DataRow)) as DataRow[];
+
+			DataRow[] result = relation.ParentTable.NewRowArray(rows.Count);
+			rows.CopyTo(result, 0);
+			return result;
 		}
 
 		/// <summary>
@@ -682,12 +930,22 @@ namespace System.Data {
 			switch (version)
 			{
 				case DataRowVersion.Default:
+					if (rowState == DataRowState.Deleted)
+						return false;
+					if (rowState == DataRowState.Detached)
+						return proposed != null;
 					return true;
 				case DataRowVersion.Proposed:
+					if (rowState == DataRowState.Deleted)
+						return false;
 					return (proposed != null);
 				case DataRowVersion.Current:
+					if (rowState == DataRowState.Deleted || rowState == DataRowState.Detached)
+						return false;
 					return (current != null);
 				case DataRowVersion.Original:
+					if (rowState == DataRowState.Detached)
+						return false;
 					return (original != null);
 			}
 			return false;
@@ -736,6 +994,8 @@ namespace System.Data {
 		/// </summary>
 		public void RejectChanges () 
 		{
+			if (RowState == DataRowState.Detached)
+				throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
 			// If original is null, then nothing has happened since AcceptChanges
 			// was last called.  We have no "original" to go back to.
 			if (original != null)
@@ -747,7 +1007,7 @@ namespace System.Data {
 				switch (rowState)
 				{
 					case DataRowState.Added:
-						_table.Rows.Remove (this);
+						_table.Rows.RemoveInternal (this);
 						break;
 					case DataRowState.Modified:
 						rowState = DataRowState.Unchanged;
@@ -765,7 +1025,7 @@ namespace System.Data {
 				// if so: FIXME ;)
 				
 				if ((rowState & DataRowState.Added) > 0)
-					_table.Rows.Remove (this);
+					_table.Rows.RemoveInternal (this);
 			}
 		}
 
@@ -809,7 +1069,7 @@ namespace System.Data {
 		[MonoTODO]
 		public void SetParentRow (DataRow parentRow) 
 		{
-			throw new NotImplementedException ();
+			SetParentRow(parentRow, null);
 		}
 
 		/// <summary>
@@ -819,12 +1079,50 @@ namespace System.Data {
 		[MonoTODO]
 		public void SetParentRow (DataRow parentRow, DataRelation relation) 
 		{
-			throw new NotImplementedException ();
+			if (_table == null || parentRow.Table == null || RowState == DataRowState.Detached)
+				throw new RowNotInTableException("This row has been removed from a table and does not have any data.  BeginEdit() will allow creation of new data in this row.");
+
+			if (parentRow != null && _table.DataSet != parentRow.Table.DataSet)
+				throw new ArgumentException();
+			
+			BeginEdit();
+			if (relation == null)
+			{
+				foreach (DataRelation parentRel in _table.ParentRelations)
+				{
+					DataColumn[] childCols = parentRel.ChildKeyConstraint.Columns;
+					DataColumn[] parentCols = parentRel.ChildKeyConstraint.RelatedColumns;
+					
+					for (int i = 0; i < parentCols.Length; i++)
+					{
+						if (parentRow == null)
+							this[childCols[i].Ordinal] = DBNull.Value;
+						else
+							this[childCols[i].Ordinal] = parentRow[parentCols[i]];
+					}
+					
+				}
+			}
+			else
+			{
+				DataColumn[] childCols = relation.ChildKeyConstraint.Columns;
+				DataColumn[] parentCols = relation.ChildKeyConstraint.RelatedColumns;
+					
+				for (int i = 0; i < parentCols.Length; i++)
+				{
+					if (parentRow == null)
+						this[childCols[i].Ordinal] = DBNull.Value;
+					else
+						this[childCols[i].Ordinal] = parentRow[parentCols[i]];
+				}
+			}
+			EndEdit();
 		}
 		
 		//Copy all values of this DataaRow to the row parameter.
 		internal void CopyValuesToRow(DataRow row)
 		{
+						
 			if (row == null)
 				throw new ArgumentNullException("row");
 			if (row == this)
@@ -833,9 +1131,10 @@ namespace System.Data {
 			DataColumnCollection columns = Table.Columns;
 			
 			for(int i = 0; i < columns.Count; i++){
+
 				string columnName = columns[i].ColumnName;
 				int index = row.Table.Columns.IndexOf(columnName);
-				//if a column withe the same name exists in bote rows copy the values
+				//if a column with the same name exists in both rows copy the values
 				if(index != -1) {
 					if (HasVersion(DataRowVersion.Original))
 					{
@@ -855,6 +1154,10 @@ namespace System.Data {
 							row.proposed = new object[row.Table.Columns.Count];
 						row.proposed[index] = row.SetColumnValue(proposed[i], index);
 					}
+					
+					//Saving the current value as the column value
+					row[index] = row.current[index];
+					
 				}
 			}
 
@@ -872,21 +1175,26 @@ namespace System.Data {
 
 			if (args.Action == System.ComponentModel.CollectionChangeAction.Add)
 			{
-				object[] tmp = new object[current.Length + 1];
-				Array.Copy (current, tmp, current.Length);
-				tmp[tmp.Length - 1] = DBNull.Value;
-				current = tmp;
-
+				object[] tmp;
+				if (current != null)
+				{
+					tmp = new object[current.Length + 1];
+					Array.Copy (current, tmp, current.Length);
+					tmp[tmp.Length - 1] = DBNull.Value;
+					current = tmp;
+				}
 				if (proposed != null)
 				{
 					tmp = new object[proposed.Length + 1];
 					Array.Copy (proposed, tmp, proposed.Length);
+					tmp[tmp.Length - 1] = DBNull.Value;
 					proposed = tmp;
 				}
 				if(original != null)
 				{
 					tmp = new object[original.Length + 1];
 					Array.Copy (original, tmp, original.Length);
+					tmp[tmp.Length - 1] = DBNull.Value;
 					original = tmp;
 				}
 
@@ -916,6 +1224,31 @@ namespace System.Data {
 			return false;
 		}
 
+		internal bool HasParentCollection
+		{
+			get
+			{
+				return _hasParentCollection;
+			}
+			set
+			{
+				_hasParentCollection = value;
+			}
+		}
+
+		internal void CheckNullConstraints()
+		{
+			if (_nullConstraintViolation)
+			{
+				for (int i = 0; i < proposed.Length; i++)
+				{
+					if (this[i] == DBNull.Value && !_table.Columns[i].AllowDBNull)
+						throw new NoNullAllowedException(_nullConstraintMessage);
+				}
+				_nullConstraintViolation = false;
+			}
+		}
+
 		#endregion // Methods
 	}
 
