@@ -11,7 +11,7 @@
 //
 // (C) Chris Podurgiel
 // (C) Ximian, Inc 2002
-// Copyright (C) Tim Coleman, 2002
+// Copyright (C) Tim Coleman, 2002-2003
 // Copyright (C) Daniel Morgan, 2002-2003
 //
 
@@ -19,7 +19,9 @@ using System;
 using System.Collections;
 using System.ComponentModel;
 using System.Globalization;
+using System.IO;
 using System.Runtime.Serialization;
+using System.Xml;
 
 namespace System.Data {
 	//[Designer]
@@ -52,6 +54,9 @@ namespace System.Data {
 		private string _tableName;
 		private bool _containsListCollection;
 		private string _encodedTableName;
+		internal bool _duringDataLoad;
+		private bool dataSetPrevEnforceConstraints;
+
 
 		
 		// If CaseSensitive property is changed once it does not anymore follow owner DataSet's 
@@ -75,8 +80,7 @@ namespace System.Data {
 			_primaryKey = null;
 			_site = null;
 			_rows = new DataRowCollection (this);
-			_locale = CultureInfo.CurrentCulture;
-
+			
 			//LAMESPEC: spec says 25 impl does 50
 			_minimumCapacity = 50;
 			
@@ -107,6 +111,14 @@ namespace System.Data {
 			//
 		}
 
+#if NET_1_2
+		public DataTable (string tableName, string tbNamespace)
+			: this (tableName)
+		{
+			_nameSpace = tbNamespace;
+		}
+#endif
+
 		/// <summary>
 		/// Indicates whether string comparisons within the table are case-sensitive.
 		/// </summary>
@@ -142,12 +154,24 @@ namespace System.Data {
 			OnRowDeleted (e);
 		}
 
+		internal void DeletingDataRow (DataRow dr, DataRowAction action) 
+		{
+			DataRowChangeEventArgs e = new DataRowChangeEventArgs (dr, action);
+			OnRowDeleting(e);
+		}
+
 		internal void ChangedDataRow (DataRow dr, DataRowAction action) 
 		{
 			DataRowChangeEventArgs e = new DataRowChangeEventArgs (dr, action);
 			OnRowChanged (e);
 		}
 
+		internal void ChangingDataRow (DataRow dr, DataRowAction action) 
+		{
+			DataRowChangeEventArgs e = new DataRowChangeEventArgs (dr, action);
+			OnRowChanging (e);
+		}
+
 		/// <summary>
 		/// Gets the collection of child relations for this DataTable.
 		/// </summary>
@@ -234,7 +258,15 @@ namespace System.Data {
 		[Browsable (false)]
 		[DataSysDescription ("Returns whether the table has errors.")]
 		public bool HasErrors {
-			get { return _hasErrors; }
+			get { 
+				// we can not use the _hasError flag because we do not know when to turn it off!
+				for (int i = 0; i < _rows.Count; i++)
+				{
+					if (_rows[i].HasErrors)
+						return true;
+				}
+				return false;
+			}
 		}
 
 		/// <summary>
@@ -243,8 +275,21 @@ namespace System.Data {
 		/// </summary>
 		[DataSysDescription ("Indicates a locale under which to compare strings within the table.")]
 		public CultureInfo Locale {
-			get { return _locale; }
-			set { _locale = value; }
+			get { 
+				// if the locale is null, we check for the DataSet locale
+				// and if the DataSet is null we return the current culture.
+				// this way if DataSet locale is changed, only if there is no locale for 
+				// the DataTable it influece the Locale get;
+				if (_locale != null)
+					return _locale;
+				if (DataSet != null)
+					return DataSet.Locale;
+				return CultureInfo.CurrentCulture;
+			}
+			set { 
+				if (_locale == null || !_locale.Equals(value))
+					_locale = value; 
+			}
 		}
 
 		/// <summary>
@@ -307,43 +352,47 @@ namespace System.Data {
 				return uc.Columns;
 			}
 			set {
-
-				//YUK: msft removes a previous unique constraint if it is flagged as a pk  
-				//when a new pk is set 
+				UniqueConstraint oldPKConstraint = UniqueConstraint.GetPrimaryKeyConstraint( Constraints);
 				
-				//clear Primary Key if value == null
-				if (null == value) {
-					
-					RemoveUniqueConstraints ();
+				// first check if value is the same as current PK.
+				if (oldPKConstraint != null && DataColumn.AreColumnSetsTheSame(value, oldPKConstraint.Columns))
 					return;
+
+				// remove PK Constraint
+				if(oldPKConstraint != null)
+				{
+					Constraints.Remove(oldPKConstraint);
 				}
+				
+				if (value != null)
+				{
+					//Does constraint exist for these columns
+					UniqueConstraint uc = UniqueConstraint.GetUniqueConstraintForColumnSet(
+						this.Constraints, (DataColumn[]) value);
+				
+					//if constraint doesn't exist for columns
+					//create new unique primary key constraint
+					if (null == uc) 
+					{
+						foreach (DataColumn Col in (DataColumn[]) value) 
+						{
 
-				//Does constraint exist for these columns
-				UniqueConstraint uc = UniqueConstraint.GetUniqueConstraintForColumnSet(
-					this.Constraints, (DataColumn[]) value);
+							if (Col.Table == null)
+								break;
 
-				//if constraint doesn't exist for columns
-				//create new unique primary key constraint
-				if (null == uc) {
+							if (Columns.IndexOf (Col) < 0)
+								throw new ArgumentException ("PrimaryKey columns do not belong to this table.");
+						}
 
-					RemoveUniqueConstraints ();						
+
+						uc = new UniqueConstraint( (DataColumn[]) value, true);
 					
-					foreach (DataColumn Col in (DataColumn[]) value) {
-
-						if (Col.Table == null)
-							break;
-
-						if (Columns.IndexOf (Col) < 0)
-							throw new ArgumentException ("PrimaryKey columns do not belong to this table.");
+						Constraints.Add (uc);
+					}
+					else 
+					{ //set existing constraint as the new primary key
+						UniqueConstraint.SetAsPrimaryKey(this.Constraints, uc);
 					}
-
-
-					uc = new UniqueConstraint( (DataColumn[]) value, true);
-					
-					Constraints.Add (uc);
-				}
-				else { //set existing constraint as the new primary key
-					UniqueConstraint.SetAsPrimaryKey(this.Constraints, uc);
 				}
 				
 			}
@@ -396,9 +445,19 @@ namespace System.Data {
 		{
 			//FIXME: Do we need to validate anything here or
 			//try to catch any errors to deal with them?
-
-			foreach(DataRow myRow in _rows) {
+			
+			// we do not use foreach because if one of the rows is in Delete state
+			// it will be romeved from Rows and we get an exception.
+			DataRow myRow;
+			for (int i = 0; i < Rows.Count; )
+			{
+				myRow = Rows[i];
 				myRow.AcceptChanges();
+
+				// if the row state is Detached it meens that it was removed from row list (Rows)
+				// so we should not increase 'i'.
+				if (myRow.RowState != DataRowState.Detached)
+					i++;
 			}
 		}
 
@@ -418,6 +477,21 @@ namespace System.Data {
 		[MonoTODO]
 		public void BeginLoadData () 
 		{
+			if (!this._duringDataLoad)
+			{
+				//duringDataLoad is important to EndLoadData and
+				//for not throwing unexpected exceptions.
+				this._duringDataLoad = true;
+			
+				if (this.dataSet != null)
+				{
+					//Saving old Enforce constraints state for later
+					//use in the EndLoadData.
+					this.dataSetPrevEnforceConstraints = this.dataSet.EnforceConstraints;
+					this.dataSet.EnforceConstraints = false;
+				}
+			}
+			return;
 		}
 
 		/// <summary>
@@ -427,9 +501,33 @@ namespace System.Data {
 			// TODO: thow an exception if any rows that 
 			//       have enforced child relations 
 			//       that would result in child rows being orphaned
+			// now we check if any ForeignKeyConstraint is referncing 'table'.
+			if (DataSet != null)
+			{
+				IEnumerator tableEnumerator = DataSet.Tables.GetEnumerator();
+			
+				// loop on all tables in dataset
+				while (tableEnumerator.MoveNext())
+				{
+					IEnumerator constraintEnumerator = ((DataTable) tableEnumerator.Current).Constraints.GetEnumerator();
+					// loop on all constrains in the current table
+					while (constraintEnumerator.MoveNext())
+					{
+						Object o = constraintEnumerator.Current;
+						// we only check ForeignKeyConstraint
+						if (o is ForeignKeyConstraint)
+						{
+							ForeignKeyConstraint fc = (ForeignKeyConstraint) o;
+							if(fc.RelatedTable == this && fc.Table.Rows.Count > 0)
+								throw new InvalidConstraintException("Cannot clear table " + fc.RelatedTable + " because ForeignKeyConstraint " + fc.ConstraintName + " enforces constraints and there are child rows in " + fc.Table);
+						}
+					}
+				}
+			}
+			
 			// TODO: throw a NotSupportedException if the DataTable is part
 			//       of a DataSet that is binded to an XmlDataDocument
-
+			
 			_rows.Clear ();
 		}
 
@@ -441,6 +539,7 @@ namespace System.Data {
 		public virtual DataTable Clone () 
 		{
 			DataTable Copy = new DataTable ();			
+			
 			CopyProperties (Copy);
 			return Copy;
 		}
@@ -460,21 +559,11 @@ namespace System.Data {
 
 			object obj = null;
 
-			// filter rows
-			ExpressionElement Expression = new ExpressionMainElement (filter);
+			DataRow[] rows = Select(filter);
 			
-			ArrayList List = new ArrayList ();
-			foreach (DataRow Row in Rows) {
-				
-				if (Expression.Test (Row))
-					List.Add (Row);
-			}
+			ExpressionAggregate Expression = new ExpressionAggregate (expression);
+			obj = Expression.Result(rows);
 			
-			DataRow[] rows = (DataRow [])List.ToArray (typeof (DataRow));
-
-			// TODO: with the filtered rows, execute the aggregate function
-			//       mentioned in expression
-
 			return obj;
 		}
 
@@ -487,21 +576,22 @@ namespace System.Data {
 			DataTable Copy = new DataTable ();
 			CopyProperties (Copy);
 
+			// we can not simply copy the row values (NewRow [C.ColumnName] = Row [C.ColumnName])
+			// because if the row state is deleted we get an exception.
+			Copy._duringDataLoad = true;
 			foreach (DataRow Row in Rows) {
 				DataRow NewRow = Copy.NewRow ();
-				NewRow.RowError = Row.RowError;
-				foreach (DataColumn C in Copy.Columns) {
-					NewRow [C.ColumnName] = Row [C.ColumnName];
-				}
 				Copy.Rows.Add (NewRow);
+				Row.CopyValuesToRow(NewRow);
 			}
-		       			
+		       	Copy._duringDataLoad = false;		
 			return Copy;
 		}
 
 		[MonoTODO]
 		private void CopyProperties (DataTable Copy) 
 		{
+					
 			Copy.CaseSensitive = CaseSensitive;
 			Copy.VirginCaseSensitive = VirginCaseSensitive;
 
@@ -517,15 +607,46 @@ namespace System.Data {
 			Copy.Namespace = Namespace;
 			// Copy.ParentRelations
 			Copy.Prefix = Prefix;
-			//Copy.PrimaryKey = PrimaryKey;
 			Copy.Site = Site;
 			Copy.TableName = TableName;
 
+
+
 			// Copy columns
 			foreach (DataColumn Column in Columns) {
-				Copy.Columns.Add (CopyColumn (Column));				
+				
+				Copy.Columns.Add (CopyColumn (Column));	
 			}
 
+			CopyConstraints(Copy);
+			// add primary key to the copy
+			if (PrimaryKey.Length > 0)
+			{
+				DataColumn[] pColumns = new DataColumn[PrimaryKey.Length];
+				for (int i = 0; i < pColumns.Length; i++)
+					pColumns[i] = Copy.Columns[PrimaryKey[i].ColumnName];
+
+				Copy.PrimaryKey = pColumns;
+			}
+		}
+
+		private void CopyConstraints(DataTable copy)
+		{
+			UniqueConstraint origUc;
+			UniqueConstraint copyUc;
+			for (int i = 0; i < this.Constraints.Count; i++)
+			{
+				if (this.Constraints[i] is UniqueConstraint)
+				{
+					origUc = (UniqueConstraint)this.Constraints[i];
+					DataColumn[] columns = new DataColumn[origUc.Columns.Length];
+					for (int j = 0; j < columns.Length; j++)
+						columns[j] = copy.Columns[origUc.Columns[j].ColumnName];
+					
+					copyUc = new UniqueConstraint(origUc.ConstraintName, columns, origUc.IsPrimaryKey);
+					copy.Constraints.Add(copyUc);
+				}
+			}
 		}
 		/// <summary>
 		/// Ends the initialization of a DataTable that is used 
@@ -545,6 +666,28 @@ namespace System.Data {
 		[MonoTODO]
 		public void EndLoadData() 
 		{
+			int i = 0;
+			if (this._duringDataLoad) 
+			{
+				//Returning from loading mode, raising exceptions as usual
+				this._duringDataLoad = false;
+				
+				if (this.dataSet !=null)
+				{
+					//Getting back to previous EnforceConstraint state
+					this.dataSet.EnforceConstraints = this.dataSetPrevEnforceConstraints;
+				}
+				for (i=0 ; i<this.Rows.Count ; i++)
+				{
+					if (this.Rows[i]._nullConstraintViolation )
+					{
+						throw new ConstraintException ("Failed to enable constraints. One or more rows contain values violating non-null, unique, or foreign-key constraints.");
+					}
+						
+				}
+
+			}
+
 		}
 
 		/// <summary>
@@ -555,8 +698,7 @@ namespace System.Data {
 		[MonoTODO]
 		public DataTable GetChanges() 
 		{
-			//TODO:
-			return this;
+			return GetChanges(DataRowState.Added | DataRowState.Deleted | DataRowState.Modified);
 		}
 
 		/// <summary>
@@ -567,17 +709,45 @@ namespace System.Data {
 		[MonoTODO]	
 		public DataTable GetChanges(DataRowState rowStates) 
 		{
-			//TODO:
-			return this;
+			DataTable copyTable = null;
+
+			IEnumerator rowEnumerator = Rows.GetEnumerator();
+			while (rowEnumerator.MoveNext()) {
+				DataRow row = (DataRow)rowEnumerator.Current;
+				if (row.IsRowChanged(rowStates)) {
+					if (copyTable == null)
+						copyTable = Clone();
+					DataRow newRow = copyTable.NewRow();
+					copyTable.Rows.Add(newRow);
+					row.CopyValuesToRow(newRow);
+				}
+			}
+			 
+			return copyTable;
 		}
 
+#if NET_1_2
+		[MonoTODO]
+		public DataTableReader GetDataReader ()
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		/// <summary>
 		/// Gets an array of DataRow objects that contain errors.
 		/// </summary>
 		[MonoTODO]
 		public DataRow[] GetErrors () 
 		{
-			throw new NotImplementedException ();
+			ArrayList errors = new ArrayList();
+			for (int i = 0; i < _rows.Count; i++)
+			{
+				if (_rows[i].HasErrors)
+					errors.Add(_rows[i]);
+			}
+			
+			return (DataRow[]) errors.ToArray(typeof(DataRow));
 		}
 	
 		/// <summary>
@@ -617,6 +787,10 @@ namespace System.Data {
 		[MonoTODO]
 		public void ImportRow (DataRow row) 
 		{
+			DataRow newRow = NewRow();
+			Rows.Add(newRow);
+			row.CopyValuesToRow(newRow);
+			
 		}
 
 		/// <summary>
@@ -627,6 +801,20 @@ namespace System.Data {
 		{
 		}
 
+#if NET_1_2
+		[MonoTODO]
+		public void Load (IDataReader reader)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void Load (IDataReader reader, LoadOption loadOption)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		/// <summary>
 		/// Finds and updates a specific row. If no matching row
 		///  is found, a new row is created using the given values.
@@ -640,11 +828,61 @@ namespace System.Data {
 				if (fAcceptChanges)
 					row.AcceptChanges ();
 			}
-			else
-				throw new NotImplementedException ();
+			else {
+				bool hasPrimaryValues = true;
+				// initiate an array that has the values of the primary keys.
+				object[] keyValues = new object[PrimaryKey.Length];
+				for (int i = 0; i < keyValues.Length && hasPrimaryValues; i++)
+				{
+					if(PrimaryKey[i].Ordinal < values.Length)
+						keyValues[i] = values[PrimaryKey[i].Ordinal];
+					else
+						hasPrimaryValues = false;
+				}
+				
+				if (hasPrimaryValues){
+					// find the row in the table.
+					row = Rows.Find(keyValues);
+				}
+				
+				if (row == null)
+					row = Rows.Add (values);
+				else
+					row.ItemArray = values;
+				
+				if (fAcceptChanges)
+					row.AcceptChanges ();
+			}
+				
 			return row;
 		}
 
+#if NET_1_2
+		[MonoTODO]
+		public DataRow LoadDataRow (object[] values, LoadOption loadOption)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void Merge (DataTable table)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void Merge (DataTable table, bool preserveChanges)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void Merge (DataTable table, bool preserveChanges, MissingSchemaAction missingSchemaAction)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		/// <summary>
 		/// Creates a new DataRow with the same schema as the table.
 		/// </summary>
@@ -669,7 +907,38 @@ namespace System.Data {
 		{
 			return new DataRow (builder);
 		}
-	
+
+#if NET_1_2
+		[MonoTODO]
+		XmlReadMode ReadXml (Stream stream)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void ReadXmlSchema (Stream stream)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void ReadXmlSchema (TextReader reader)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void ReadXmlSchema (string fileName)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void ReadXmlSchema (XmlReader reader)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 
 		/// <summary>
 		/// Rolls back all changes that have been made to the 
@@ -679,8 +948,6 @@ namespace System.Data {
 		[MonoTODO]
 		public void RejectChanges () 
 		{	
-			//foreach(DataRow myRow in _rows)
-			//{
 			for (int i = _rows.Count - 1; i >= 0; i--) {
 				DataRow row = _rows [i];
 				if (row.RowState != DataRowState.Unchanged)
@@ -694,6 +961,20 @@ namespace System.Data {
 		[MonoTODO]
 		public virtual void Reset () 
 		{
+			Clear();
+			while (ParentRelations.Count > 0)
+			{
+				if (dataSet.Relations.Contains(ParentRelations[ParentRelations.Count - 1].RelationName))
+					dataSet.Relations.Remove(ParentRelations[ParentRelations.Count - 1]);
+			}
+
+			while (ChildRelations.Count > 0)
+			{
+				if (dataSet.Relations.Contains(ChildRelations[ChildRelations.Count - 1].RelationName))
+					dataSet.Relations.Remove(ChildRelations[ChildRelations.Count - 1]);
+			}
+			Constraints.Clear();
+			Columns.Clear();
 		}
 
 		/// <summary>
@@ -701,9 +982,7 @@ namespace System.Data {
 		/// </summary>
 		public DataRow[] Select () 
 		{
-			DataRow[] dataRows = new DataRow[_rows.Count];
-			_rows.CopyTo (dataRows, 0);
-			return dataRows;
+			return Select(String.Empty, String.Empty, DataViewRowState.CurrentRows);
 		}
 
 		/// <summary>
@@ -713,16 +992,7 @@ namespace System.Data {
 		/// </summary>
 		public DataRow[] Select (string filterExpression) 
 		{
-			ExpressionElement Expression = new ExpressionMainElement (filterExpression);
-
-			ArrayList List = new ArrayList ();
-			foreach (DataRow Row in Rows) {
-				
-				if (Expression.Test (Row))
-					List.Add (Row);
-			}
-			
-			return (DataRow [])List.ToArray (typeof (DataRow));
+			return Select(filterExpression, String.Empty, DataViewRowState.CurrentRows);
 		}
 
 		/// <summary>
@@ -732,30 +1002,7 @@ namespace System.Data {
 		/// </summary>
 		public DataRow[] Select (string filterExpression, string sort) 
 		{
-			DataRow[] dataRows = null;
-
-			if (filterExpression != null && filterExpression.Equals (String.Empty) == false)
-				dataRows = Select (filterExpression);
-			else
-				dataRows = Select ();
-
-			if (sort != null && !sort.Equals (String.Empty)) {
-				SortableColumn[] sortableColumns = null;
-
-				sortableColumns = ParseTheSortString (sort);
-				if (sortableColumns == null)
-					throw new Exception ("sort expression result is null");
-				if (sortableColumns.Length == 0)
-					throw new Exception("sort expression result is 0");
-
-				RowSorter rowSorter = new RowSorter (dataRows, sortableColumns);
-				dataRows = rowSorter.SortRows ();
-			
-				sortableColumns = null;
-				rowSorter = null;
-			}
-
-			return dataRows;
+			return Select(filterExpression, sort, DataViewRowState.CurrentRows);
 		}
 
 		/// <summary>
@@ -767,13 +1014,66 @@ namespace System.Data {
 		public DataRow[] Select(string filterExpression, string sort, DataViewRowState recordStates) 
 		{
 			DataRow[] dataRows = null;
+			if (filterExpression == null)
+				filterExpression = String.Empty;
 
-			// TODO: do something with recordStates
-			dataRows = Select (filterExpression, sort);
+			ExpressionElement Expression = null;
+			if (filterExpression != null && filterExpression.Length != 0)
+				Expression = new ExpressionMainElement(filterExpression);
 
+			ArrayList List = new ArrayList();
+			int recordStateFilter = GetRowStateFilter(recordStates);
+			foreach (DataRow Row in Rows) 
+			{
+				if (((int)Row.RowState & recordStateFilter) != 0)
+				{
+					if (Expression == null || Expression.Test(Row))
+						List.Add(Row);
+				}
+			}
+
+			dataRows = (DataRow[])List.ToArray(typeof(DataRow));
+			
+
+			if (sort != null && !sort.Equals(String.Empty)) 
+			{
+				SortableColumn[] sortableColumns = null;
+
+				sortableColumns = ParseTheSortString (sort);
+				if (sortableColumns == null)
+					throw new Exception ("sort expression result is null");
+				if (sortableColumns.Length == 0)
+					throw new Exception("sort expression result is 0");
+
+				RowSorter rowSorter = new RowSorter (dataRows, sortableColumns);
+				dataRows = rowSorter.SortRows ();
+
+				sortableColumns = null;
+				rowSorter = null;
+			}
+
+			
 			return dataRows;
 		}
 
+		private static int GetRowStateFilter(DataViewRowState recordStates)
+		{
+			int flag = 0;
+
+			if ((recordStates & DataViewRowState.Added) != 0)
+				flag |= (int)DataRowState.Added;
+			if ((recordStates & DataViewRowState.Deleted) != 0)
+				flag |= (int)DataRowState.Deleted;
+			if ((recordStates & DataViewRowState.ModifiedCurrent) != 0)
+				flag |= (int)DataRowState.Modified;
+			if ((recordStates & DataViewRowState.ModifiedOriginal) != 0)
+				flag |= (int)DataRowState.Modified;
+			if ((recordStates & DataViewRowState.Unchanged) != 0)
+				flag |= (int)DataRowState.Unchanged;
+
+			return flag;
+		}
+
 		/// <summary>
 		/// Gets the TableName and DisplayExpression, if 
 		/// there is one as a concatenated string.
@@ -782,9 +1082,85 @@ namespace System.Data {
 		{
 			//LAMESPEC: spec says concat the two. impl puts a 
 			//plus sign infront of DisplayExpression
-			return TableName + " + " + DisplayExpression;
+			string retVal = TableName;
+			if(DisplayExpression != null && DisplayExpression != "")
+				retVal += " + " + DisplayExpression;
+			return retVal;
 		}
 
+#if NET_1_2
+		[MonoTODO]
+		public void WriteXml (Stream stream)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (TextWriter writer)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (XmlWriter writer)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (string fileName)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (Stream stream, XmlWriteMode mode)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (TextWriter writer, XmlWriteMode mode)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (XmlWriter writer, XmlWriteMode mode)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXml (string fileName, XmlWriteMode mode)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXmlSchema (Stream stream)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXmlSchema (TextWriter writer)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXmlSchema (XmlWriter writer)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public void WriteXmlSchema (string fileName)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 		
 		#region Events /////////////////
 		
@@ -877,9 +1253,9 @@ namespace System.Data {
 			Copy.Caption = Column.Caption;
 			Copy.ColumnMapping = Column.ColumnMapping;
 			Copy.ColumnName = Column.ColumnName;
-			// Copy.Container
-			// Copy.DataType
-			// Copy.DefaultValue			
+			//Copy.Container
+			Copy.DataType = Column.DataType;
+			Copy.DefaultValue = Column.DefaultValue;			
 			Copy.Expression = Column.Expression;
 			//Copy.ExtendedProperties
 			Copy.MaxLength = Column.MaxLength;
@@ -887,7 +1263,8 @@ namespace System.Data {
 			Copy.Prefix = Column.Prefix;
 			Copy.ReadOnly = Column.ReadOnly;
 			//Copy.Site
-			Copy.Unique = Column.Unique;
+			//we do not copy the unique value - it will be copyied when copying the constraints.
+			//Copy.Unique = Column.Unique;
 			
 			return Copy;
 		}			
